package braze.com.brazepluginexample

import android.os.Bundle
import android.view.View

import io.flutter.app.FlutterActivity
import io.flutter.plugins.GeneratedPluginRegistrant

import com.appboy.Appboy
import com.appboy.AppboyLifecycleCallbackListener
import com.appboy.events.ContentCardsUpdatedEvent
import com.appboy.events.IEventSubscriber
import com.appboy.models.cards.Card
import com.appboy.models.IInAppMessage
import com.appboy.models.IInAppMessageImmersive
import com.appboy.models.MessageButton
import com.appboy.ui.inappmessage.AppboyInAppMessageManager
import com.appboy.ui.inappmessage.InAppMessageCloser
import com.appboy.ui.inappmessage.InAppMessageOperation
import com.appboy.ui.inappmessage.listeners.IInAppMessageManagerListener


import com.braze.brazeplugin.BrazePlugin

class MainActivity: FlutterActivity() {
  private var mContentCardsUpdatedSubscriber: IEventSubscriber<ContentCardsUpdatedEvent>? = null
  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    GeneratedPluginRegistrant.registerWith(this)
    this.getApplication().registerActivityLifecycleCallbacks(AppboyLifecycleCallbackListener())
    this.subscribeToContentCardsUpdatedEvent()

    class BrazeInAppMessageManagerListener() : IInAppMessageManagerListener {
      override fun onInAppMessageReceived(inAppMessage: IInAppMessage): Boolean {
        return false
      }

      override fun beforeInAppMessageDisplayed(inAppMessage: IInAppMessage): InAppMessageOperation {
        BrazePlugin.processInAppMessage(inAppMessage)
        return InAppMessageOperation.DISPLAY_NOW
      }

      override fun onInAppMessageClicked(inAppMessage: IInAppMessage,
                                         inAppMessageCloser: InAppMessageCloser): Boolean {
        return false
      }

      override fun onInAppMessageButtonClicked(inAppMessage: IInAppMessage, button: MessageButton,
                                               inAppMessageCloser: InAppMessageCloser): Boolean {
        return false
      }

      override fun onInAppMessageDismissed(inAppMessage: IInAppMessage) {}

      override fun beforeInAppMessageViewClosed(inAppMessageView: View, inAppMessage: IInAppMessage) {}

      override fun afterInAppMessageViewClosed(inAppMessage: IInAppMessage) {}

      override fun beforeInAppMessageViewOpened(inAppMessageView: View, inAppMessage: IInAppMessage) {}

      override fun afterInAppMessageViewOpened(inAppMessageView: View, inAppMessage: IInAppMessage) {}
    }
    AppboyInAppMessageManager.getInstance().setCustomInAppMessageManagerListener(
        BrazeInAppMessageManagerListener())

    Appboy.getInstance(this).logCustomEvent("flutter_sample_opened")

  }
  private fun subscribeToContentCardsUpdatedEvent() {
    Appboy.getInstance(this).removeSingleSubscription(mContentCardsUpdatedSubscriber, ContentCardsUpdatedEvent::class.java)
    mContentCardsUpdatedSubscriber = IEventSubscriber { event ->
      val allCards = event.allCards as ArrayList<Card>
      BrazePlugin.processContentCards(allCards)
    }
    Appboy.getInstance(this).subscribeToContentCardsUpdates(mContentCardsUpdatedSubscriber)
    Appboy.getInstance(this).requestContentCardsRefresh(true)
  }
}
